@@ -1,3 +1,27 @@
+// SPDX-License-Identifier: GPL-3.0-or-later
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+*
+* Copyright (C) 2013 - 2024, nymea GmbH
+* Copyright (C) 2024 - 2025, chargebyte austria GmbH
+*
+* This file is part of nymea-energy-plugin-chargingsessions.
+*
+* nymea-energy-plugin-chargingsessions is free software: you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation, either version 3 of the License, or
+* (at your option) any later version.
+*
+* nymea-energy-plugin-chargingsessions is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+* GNU General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with nymea-energy-plugin-chargingsessions. If not, see <https://www.gnu.org/licenses/>.
+*
+* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
 #include "testchargingsessionsdatabase.h"
 #include "../../../energyplugin/chargingsessionsdatabase.h"
 #include "../../../energyplugin/chargingsessionssettings.h"
@@ -5,6 +29,8 @@
 
 #include <QSignalSpy>
 #include <QDateTime>
+#include <QTemporaryDir>
+#include <QDir>
 
 ChargingSessionsDatabaseTests::ChargingSessionsDatabaseTests(QObject *parent)
     : QObject(parent)
@@ -269,4 +295,74 @@ void ChargingSessionsDatabaseTests::fullSessionLogSessionEnergy()
     QCOMPARE(session.energyEnd(), 0);
 }
 
+void ChargingSessionsDatabaseTests::fetchCarSessionsTimeRange()
+{
+    QTemporaryDir tempDir;
+    QVERIFY(tempDir.isValid());
+    const QString databaseName = tempDir.path() + QDir::separator() + QStringLiteral("chargingsessions.db");
+
+    QUuid evChargerId = QUuid::createUuid();
+    QString evChargerName = QStringLiteral("Charger");
+    QString serialNumber = QStringLiteral("SERIAL");
+    QUuid carId = QUuid::createUuid();
+    QString carName = QStringLiteral("Car");
+
+    const QDateTime baseTime = QDateTime::currentDateTime().addDays(-1);
+
+    ChargingSessionsDatabase database(databaseName);
+
+    // Session 1: baseTime .. baseTime+60
+    QSignalSpy sessionAddedSpy1(&database, &ChargingSessionsDatabase::databaseSessionAdded);
+    database.logStartSession(evChargerId, evChargerName, serialNumber, carId, carName, baseTime);
+    if (sessionAddedSpy1.count() == 0) {
+        sessionAddedSpy1.wait();
+    }
+    QCOMPARE(sessionAddedSpy1.count(), 1);
+    const int sessionId1 = sessionAddedSpy1.at(0).at(1).toInt();
+
+    QSignalSpy sessionFinishedSpy1(&database, &ChargingSessionsDatabase::databaseSessionUpdated);
+    database.logEndSession(sessionId1, carId, carName, baseTime.addSecs(60));
+    if (sessionFinishedSpy1.count() == 0) {
+        sessionFinishedSpy1.wait();
+    }
+    QCOMPARE(sessionFinishedSpy1.count(), 1);
+
+    // Session 2: baseTime+3600 .. baseTime+3660
+    QSignalSpy sessionAddedSpy2(&database, &ChargingSessionsDatabase::databaseSessionAdded);
+    database.logStartSession(evChargerId, evChargerName, serialNumber, carId, carName, baseTime.addSecs(3600));
+    if (sessionAddedSpy2.count() == 0) {
+        sessionAddedSpy2.wait();
+    }
+    QCOMPARE(sessionAddedSpy2.count(), 1);
+    const int sessionId2 = sessionAddedSpy2.at(0).at(1).toInt();
+
+    QSignalSpy sessionFinishedSpy2(&database, &ChargingSessionsDatabase::databaseSessionUpdated);
+    database.logEndSession(sessionId2, carId, carName, baseTime.addSecs(3660));
+    if (sessionFinishedSpy2.count() == 0) {
+        sessionFinishedSpy2.wait();
+    }
+    QCOMPARE(sessionFinishedSpy2.count(), 1);
+
+    // Fetch only session 2 by time range overlap
+    FetchDataReply *fetchReply = database.fetchCarSessions(carId, baseTime.addSecs(3000), baseTime.addSecs(3700));
+    QSignalSpy fetchReplySignalSpy(fetchReply, &FetchDataReply::finished);
+    if (fetchReplySignalSpy.count() == 0) {
+        fetchReplySignalSpy.wait();
+    }
+    QCOMPARE(fetchReplySignalSpy.count(), 1);
+    QCOMPARE(fetchReply->error(), ChargingSessionsManager::ChargingSessionsErrorNoError);
+    QCOMPARE(fetchReply->sessions().count(), 1);
+    QCOMPARE(fetchReply->sessions().first().sessionId().toInt(), sessionId2);
+
+    // Fetch all sessions overlapping from baseTime+30 onwards
+    FetchDataReply *fetchReply2 = database.fetchCarSessions(carId, baseTime.addSecs(30), QDateTime());
+    QSignalSpy fetchReply2SignalSpy(fetchReply2, &FetchDataReply::finished);
+    if (fetchReply2SignalSpy.count() == 0) {
+        fetchReply2SignalSpy.wait();
+    }
+    QCOMPARE(fetchReply2SignalSpy.count(), 1);
+    QCOMPARE(fetchReply2->error(), ChargingSessionsManager::ChargingSessionsErrorNoError);
+    QCOMPARE(fetchReply2->sessions().count(), 2);
+}
+
 QTEST_MAIN(ChargingSessionsDatabaseTests)
